/*
 * Copyright (c) 2005 by Romuald DELAVERGNE
 * It can be redistributed under the terms of the GNU General Public License.
*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <io.h>

void
exitOnError(char *msg) {
	perror(msg);
	exit(1);
}

int
main(int argc, char *argv[]) {
	void *buf;
	struct stat statFileIn;
	FILE *fileIn, *fileOut;
	char
		fileName[255],
		*prog = argv[0],
		*prefix = argv[3];
	unsigned long
		nbFiles, lastSize, maxSize, size, nb, nbWrited, nbRest;
	int i;

	buf = malloc(BUFSIZ);

	if (argc < 4) {
		printf("%s: too few parameters !\n", prog);
		printf("split a file into pieces\n");
		printf("usage: %s FILE SIZE PREFIX\n", prog);
		printf("  FILE  : big file to split\n");
		printf("  SIZE  : size in bytes\n");
		printf("  PREFIX: result files without suffix\n");
		exit(0);
	}

	if ((fileIn = fopen(argv[1], "rb")) == NULL) exitOnError(argv[1]);
	if (fstat(fileIn->fd, &statFileIn) == -1) exitOnError(argv[1]);
	if (sscanf(argv[2], "%lu", &maxSize) == 0) {
		fprintf(stderr, "%s: invalid size !\n", argv[2]);
		exit(1);
	}
	nbFiles = statFileIn.st_size/maxSize;
	lastSize = statFileIn.st_size%maxSize;
	if (lastSize > 0) nbFiles++; else lastSize = maxSize;
	i = 0;
	while (i < nbFiles) {
		i++;
		size = (i == nbFiles) ? lastSize: maxSize;
		sprintf(fileName, "%s.%03d", prefix, i);
		if (access(fileName, 0) == 0) {
			fprintf(stderr, "%s: file already exists !\n", fileName);
			exit(1);
		}
		if ((fileOut = fopen(fileName, "wb")) == NULL) exitOnError(fileName);
		nbWrited = 0;
		while (nbWrited < size) {
			nbRest = size - nbWrited;
			nb = (nbRest < BUFSIZ) ? nbRest: BUFSIZ;
			if (fread(buf, nb, 1, fileIn) != 1) exitOnError(prog);
			if (fwrite(buf, nb, 1, fileOut) != 1) exitOnError(prog);
			nbWrited += nb;
		}
		fclose(fileOut);
	}
	fclose(fileIn);

	return 0;
}
